package io.tswf.minecraft.blockventure.core.api.nbt

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.exception.NBTToObjectMappingException
import io.tswf.minecraft.blockventure.core.exception.ObjectToNBTMappingException
import net.minecraft.nbt.NbtElement

import java.lang.reflect.Type

@CompileStatic
class NBTObjectMapper {

    @Lazy
    static List<INBTObjectMappingProvider> nbtObjectMappingProviders = ServiceLoader.load(INBTObjectMappingProvider)
        .toList()
        .sortBy(true) { it.priority }

    static NbtElement mapToNbt(Object object, Type objectType) {
        objectType = _checkBoxing(objectType)
        def matchingProvider = nbtObjectMappingProviders.find { it.canAcceptObjectType(objectType) }
        try {
            return matchingProvider?.mapObjectToNBT(object, objectType)
        } catch (Throwable e) {
            throw new ObjectToNBTMappingException(e, matchingProvider, object, objectType)
        }
    }

    static Object mapToObject(NbtElement nbt, Type objectType, Object initialInstance = null) {
        objectType = _checkBoxing(objectType)
        def matchingProvider = nbtObjectMappingProviders.find { it.canAcceptObjectType(objectType) && it.canAcceptNbtType(nbt.class) }
        try {
            return matchingProvider?.mapNBTToObject(nbt, objectType, initialInstance)
        } catch (Throwable e) {
            throw new NBTToObjectMappingException(e, matchingProvider, nbt, objectType)
        }
    }

    static Type _checkBoxing(Type type) {
        def typeClass = type.extractClass()
        if (typeClass.isPrimitive()) {
            return typeClass.boxed
        }

        return type
    }
}
