package io.tswf.minecraft.blockventure.core.api.recipe

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.recipe.RecipeIngredientObjectParser
import io.tswf.minecraft.blockventure.core.recipe.RecipeParser
import net.minecraft.item.ItemStack
import net.minecraft.recipe.ShapedRecipe
import net.minecraft.recipe.ShapelessRecipe
import net.minecraft.recipe.SmeltingRecipe
import net.minecraft.recipe.book.CookingRecipeCategory
import net.minecraft.recipe.book.CraftingRecipeCategory
import net.minecraft.util.Identifier

import java.util.concurrent.ConcurrentHashMap

@CompileStatic
class RecipeRegistry {

    public static final int DEFAULT_SMELTING_COOK_TIME = 200
    public static CraftingRecipeCategory defaultCraftingRecipeCategory = CraftingRecipeCategory.MISC
    public static CookingRecipeCategory defaultCookingRecipeCategory = CookingRecipeCategory.MISC
    public static String defaultGroup = "pohui"

    static Map<String, SmeltingRecipe> registeredSmeltingRecipes = new ConcurrentHashMap<>()
    static Map<String, ShapedRecipe> registeredShapedRecipes = new ConcurrentHashMap<>()
    static Map<String, ShapelessRecipe> registeredShapelessRecipe = new ConcurrentHashMap<>()

    static void registerSmeltingRecipe(CookingRecipeCategory recipeCategory = defaultCookingRecipeCategory, String group = defaultGroup, String id, ItemStack result, Object ingredient, float expCount = 0, int cookTime = DEFAULT_SMELTING_COOK_TIME) {
        def recipeInput = RecipeIngredientObjectParser.parseIngredientObject(ingredient)
        def smeltingRecipe = new SmeltingRecipe(new Identifier(id), group, recipeCategory, recipeInput, result.copy(), expCount, cookTime)

        registeredSmeltingRecipes[id] = smeltingRecipe
    }

    static void registerShapelessRecipe(CraftingRecipeCategory recipeCategory = defaultCraftingRecipeCategory, String group = defaultGroup, String id, ItemStack result, Object... ingredients) {
        def recipeIngredients = RecipeParser.parseShapelessRecipe(ingredients)
        def shapelessRecipe = new ShapelessRecipe(new Identifier(id), group, recipeCategory, result.copy(), recipeIngredients)

        registeredShapelessRecipe[id] = shapelessRecipe
    }

    static void registerShapedRecipe(CraftingRecipeCategory recipeCategory = defaultCraftingRecipeCategory, String group = defaultGroup, String id, ItemStack result, Object... ingredients) {
        def (recipeIngredients, recipeWidth, recipeHeight) = RecipeParser.parseShapedRecipePattern(ingredients)
        def shapedRecipe = new ShapedRecipe(new Identifier(id), group, recipeCategory, recipeWidth, recipeHeight, recipeIngredients, result.copy())

        registeredShapedRecipes[id] = shapedRecipe
    }
}
