package io.tswf.minecraft.blockventure.core.api.registry


import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import io.tswf.minecraft.blockventure.core.api.block.entity.IAutoTickableBlockEntity
import io.tswf.minecraft.blockventure.core.block.entity.AutoBlockEntityType
import io.tswf.minecraft.blockventure.core.block.entity.BlockEntityCreator
import io.tswf.minecraft.blockventure.core.transmutation.expansion.IAutoTickableBlockEntityType
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewBlockEntityType
import net.minecraft.block.Block
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.datafixer.TypeReferences
import net.minecraft.registry.Registries
import net.minecraft.util.Util

@CompileStatic
class BlockEntityRegistry {

    static Map<String, BlockEntityType> registeredBlockEntityTypesById = new HashMap<>()
    static Map<String, BlockEntityType> registeredBlockEntityTypesByClass = new HashMap<>()

    static BlockEntityType getAutoSelectType() {
        return AutoBlockEntityType.autoEntityType
    }

    static void registerAdditionalBlockToBlockEntityType(@NullSafe Block block, @NullSafe BlockEntityType blockEntityType) {
        ((ViewBlockEntityType) blockEntityType).blocks << block
    }

    static void registerBlockEntity(@NullSafe Block block, @NullSafe Class blockEntityClass, @NullSafe String id) {
        def blockEntityType = registeredBlockEntityTypesByClass
                .computeIfAbsent(blockEntityClass.name) {
                    def builder = BlockEntityType.Builder.create(
                            {pos, state -> BlockEntityCreator.createBlockEntity(blockEntityClass, pos, state) },
                            block
                    )

                    def type = Util.getChoiceType(TypeReferences.BLOCK_ENTITY, id);// 76
                    def result = builder.build(type)

                    if (IAutoTickableBlockEntity.isAssignableFrom(blockEntityClass)) {
                        def autoTickableBlockEntityType = (IAutoTickableBlockEntityType) result
                        autoTickableBlockEntityType.autoTickerEnabled = true
                    }

                    registeredBlockEntityTypesById[id] = result

                    Register.register(Registries.BLOCK_ENTITY_TYPE, result, id)
                }

        def registeredBlockEntityTypeId = Registries.BLOCK_ENTITY_TYPE.getId(blockEntityType)

        //TODO: Добавить учет префикса загружаемого мода
        if (registeredBlockEntityTypeId.toString() != id) {
            throw new IllegalStateException("There is already registered block entity class '${blockEntityClass.name}' with id '${registeredBlockEntityTypeId}', can not register it with new id '${id}'!")
        }

        def blockEntityTypeBlocksView = (ViewBlockEntityType) blockEntityType
        blockEntityTypeBlocksView.blocks << block
    }

    static BlockEntityType getBlockEntityTypeByClass(@NullSafe Class blockEntityClass) {
        return findBlockEntityTypeByClass(blockEntityClass).ensureNotNull { new NoSuchElementException("There is no registered block entity type with class: ${blockEntityClass.name}! Existing classes: ${registeredBlockEntityTypesByClass.keySet()}") }
    }

    static BlockEntityType findBlockEntityTypeByClass(@NullSafe Class blockEntityClass) {
        return registeredBlockEntityTypesByClass[blockEntityClass.name]
    }
}
