package io.tswf.minecraft.blockventure.core.api.world.gen.feature

import groovy.transform.CompileStatic
import net.minecraft.block.Block
import net.minecraft.block.BlockState
import net.minecraft.block.Blocks
import net.minecraft.registry.tag.BlockTags
import net.minecraft.structure.rule.BlockMatchRuleTest
import net.minecraft.structure.rule.TagMatchRuleTest
import net.minecraft.world.gen.feature.ConfiguredFeature
import net.minecraft.world.gen.feature.Feature
import net.minecraft.world.gen.feature.OreFeatureConfig
import net.minecraft.world.gen.feature.PlacedFeature
import net.minecraft.world.gen.placementmodifier.PlacementModifier

import static io.tswf.minecraft.blockventure.core.api.world.gen.feature.PlacementModifierFactory.*

@CompileStatic
class FeatureFactory {

    static PlacedFeature createSimpleOrePlacedFeature(Block oreBlock, int size, int count, int minHeight, int maxHeight, boolean deepslate = false) {
        return createSimpleOrePlacedFeature(oreBlock.getDefaultState(), size, count, minHeight, maxHeight, deepslate)
    }

    static PlacedFeature createSimpleOrePlacedFeature(BlockState oreBlockState, int size, int count, int minHeight, int maxHeight, boolean deepslate = false) {
        def oreFeatureConfig =  new OreFeatureConfig(
                new TagMatchRuleTest(deepslate ? BlockTags.DEEPSLATE_ORE_REPLACEABLES : BlockTags.STONE_ORE_REPLACEABLES), // new BlockMatchRuleTest(Blocks.STONE)
                oreBlockState,
                size
        )

        return createCountAndHeightRangePlacedFeature( new ConfiguredFeature(Feature.ORE, oreFeatureConfig), count, minHeight, maxHeight )
    }

    static PlacedFeature createCountPlacedFeature(
            ConfiguredFeature feature,
            int count,
            List<PlacementModifier> additionalModifiers = Collections.emptyList()
    ) {
        return new PlacedFeature( feature.toRegistryEntry(), [
                createCountModifier(count),
                createHorizontalRandomOffsetModifier()
        ] as List<PlacementModifier> + additionalModifiers)
    }

    static PlacedFeature createCountAndHeightRangePlacedFeature(
            ConfiguredFeature feature,
            int count,
            int minHeight,
            int maxHeight,
            List<PlacementModifier> additionalModifiers = Collections.emptyList()
    ) {
        return createCountPlacedFeature(feature, count, [ createHeightRangeModifier(minHeight, maxHeight) ] + additionalModifiers)
    }

}
