package io.tswf.minecraft.blockventure.core.block.entity

import groovy.transform.CompileStatic
import groovy.transform.Memoized
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import io.tswf.minecraft.blockventure.core.api.registry.BlockEntityRegistry
import io.tswf.minecraft.blockventure.core.exception.BlockEntityCreationException
import net.minecraft.block.BlockState
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.util.math.BlockPos

import java.lang.reflect.Constructor

@CompileStatic
class BlockEntityCreator {

    static final Class[] BLOCK_ENTITY_CONSTRUCTOR_PARAMETER_TYPES = new Class[] { BlockPos, BlockState }
    static final Class[] BLOCK_ENTITY_CONSTRUCTOR_WITH_TYPE_PARAMETER_TYPES = new Class[] { BlockEntityType, BlockPos, BlockState }

    static final Set<Class[]> BLOCK_ENTITY_CONSTRUCTORS = [ BLOCK_ENTITY_CONSTRUCTOR_PARAMETER_TYPES, BLOCK_ENTITY_CONSTRUCTOR_WITH_TYPE_PARAMETER_TYPES ] as Set

    static <T extends BlockEntity> T createBlockEntity(@NullSafe Class<T> classOfEntity, BlockPos pos, BlockState state) {
        try {
            return (T) getBlockEntityConstructor(classOfEntity)
                .ensureNotNull { new NoSuchMethodException("There is no no-args constructor in ${classOfEntity.name} found!") }
                .with {
                     // Цепляться за количество параметров, конечно, полная хуйня, а с другой стороны это просто. Возможно перепишу нормально.
                     if (parameterTypes.length == 3) {
                         newInstance(BlockEntityRegistry.getBlockEntityTypeByClass(classOfEntity), pos, state)
                     } else {
                         newInstance(pos, state)
                     }
                }

        } catch (Throwable e) {
            throw new BlockEntityCreationException(e, classOfEntity)
        }
    }

    @Memoized
    static Constructor getBlockEntityConstructor(Class blockEntityClass) {
        return blockEntityClass.constructors.stream()
            .filter {  constructor ->
                BLOCK_ENTITY_CONSTRUCTORS.any { parameterTypes ->
                    Arrays.equals(constructor.parameterTypes, parameterTypes)
                }
            }
            .reverseSortedBy { it.parameterTypes.length }
            .findFirst()
            .orElseThrow { new NoSuchMethodException("There is no required constructor in ${blockEntityClass.name} found! The block entity class must provide consturctor with some parameters set of ${ BLOCK_ENTITY_CONSTRUCTORS.map { it.map { cl -> cl.name } } }")  }
    }
}
