package io.tswf.minecraft.blockventure.core.exception

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.INBTObjectMappingProvider
import net.minecraft.nbt.NbtElement

import java.lang.reflect.Type

@CompileStatic
class NBTToObjectMappingException extends RuntimeException {
    INBTObjectMappingProvider mappingProvider
    NbtElement nbt
    Type objectType

    NBTToObjectMappingException(INBTObjectMappingProvider mappingProvider, NbtElement nbt, Type objectType) {
        super(getExceptionMessage(mappingProvider, nbt, objectType))
        this.mappingProvider = mappingProvider
        this.nbt = nbt
        this.objectType = objectType
    }

    NBTToObjectMappingException(String message, INBTObjectMappingProvider mappingProvider, NbtElement nbt, Type objectType) {
        super(getExceptionMessage(mappingProvider, nbt, objectType, message))
        this.mappingProvider = mappingProvider
        this.nbt = nbt
        this.objectType = objectType
    }

    NBTToObjectMappingException(String message, Throwable cause, INBTObjectMappingProvider mappingProvider, NbtElement nbt, Type objectType) {
        super(getExceptionMessage(mappingProvider, nbt, objectType, message), cause)
        this.mappingProvider = mappingProvider
        this.nbt = nbt
        this.objectType = objectType
    }

    NBTToObjectMappingException(Throwable cause, INBTObjectMappingProvider mappingProvider, NbtElement nbt, Type objectType) {
        super(getExceptionMessage(mappingProvider, nbt, objectType), cause)
        this.mappingProvider = mappingProvider
        this.nbt = nbt
        this.objectType = objectType
    }

    static String getExceptionMessage(INBTObjectMappingProvider mappingProvider, NbtElement nbt, Type objectType, String message = "") {
        return "Exception occurred while mapping nbt '${nbt}' to type '${objectType.typeName}' by ${mappingProvider}: ${message}"
    }
}
