package io.tswf.minecraft.blockventure.core.exception

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.INBTObjectMappingProvider

import java.lang.reflect.Type

@CompileStatic
class ObjectToNBTMappingException extends RuntimeException {
    INBTObjectMappingProvider mappingProvider
    Object object
    Type objectType

    ObjectToNBTMappingException(INBTObjectMappingProvider mappingProvider, Object object, Type objectType) {
        super(getExceptionMessage(mappingProvider, object, objectType))
        this.mappingProvider = mappingProvider
        this.object = object
        this.objectType = objectType
    }

    ObjectToNBTMappingException(String message, INBTObjectMappingProvider mappingProvider, Object object, Type objectType) {
        super(getExceptionMessage(mappingProvider, object, objectType, message))
        this.mappingProvider = mappingProvider
        this.object = object
        this.objectType = objectType
    }

    ObjectToNBTMappingException(String message, Throwable cause, INBTObjectMappingProvider mappingProvider, Object object, Type objectType) {
        super(getExceptionMessage(mappingProvider, object, objectType, message), cause)
        this.mappingProvider = mappingProvider
        this.object = object
        this.objectType = objectType
    }

    ObjectToNBTMappingException(Throwable cause, INBTObjectMappingProvider mappingProvider, Object object, Type objectType) {
        super(getExceptionMessage(mappingProvider, object, objectType), cause)
        this.mappingProvider = mappingProvider
        this.object = object
        this.objectType = objectType
    }

    static String getExceptionMessage(INBTObjectMappingProvider mappingProvider, Object object, Type objectType, String message = "") {
        return "Exception occurred while mapping object '${object}' with type '${objectType.typeName}' to NBT by ${mappingProvider}: ${message}"
    }
}
