package io.tswf.minecraft.blockventure.core.nbt.mapper

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.INBTObjectMappingProvider
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import net.minecraft.nbt.NbtElement
import net.minecraft.nbt.NbtList

import java.lang.reflect.Type
import java.util.function.Supplier

@CompileStatic
abstract class AbstractNBTCollectionMappingProvider<T extends Collection> implements INBTObjectMappingProvider {

    Class<T> collectionClass
    Supplier<T> collectionFactory

    AbstractNBTCollectionMappingProvider() {
        initialize()
        collectionFactory.ensureNotNull { new IllegalStateException("There is no collection factory was set for ${this}!") }
        collectionClass.ensureNotNull { new IllegalStateException("There is no supported collection type was set for ${this}!") }
    }

    @Override
    NbtElement mapObjectToNBT(Object obj, Type objectType) {
        def nbtList = new NbtList()
        def collection = (Collection) obj
        def collectionElementType = objectType.extractChildTypes().first()

        for (def collectionElement : collection) {
            def tag = NBTObjectMapper.mapToNbt(collectionElement, collectionElementType)
            nbtList.add(tag)
        }

        return nbtList
    }

    @Override
    Object mapNBTToObject(NbtElement nbtElement, Type objectType, Object initialInstance) {
        def nbtList = (NbtList) nbtElement
        def collectionElementType = objectType.extractChildTypes().first()
        def collection

        if (initialInstance != null) {
            if (collectionClass.isAssignableFrom(initialInstance.class)) {
                collection = (T) initialInstance
            } else {
                throw new IllegalArgumentException("Initial instance '${initialInstance}' with type ${initialInstance.class.name} is not matches with supported collection type '${collectionClass.name}'!")
            }
        } else {
            collection = collectionFactory.get()
        }

        for (def tag : nbtList) {
            def collectionElement = NBTObjectMapper.mapToObject(tag, collectionElementType)
            collection << collectionElement
        }

        return collection
    }

    @Override
    boolean canAcceptObjectType(Type objectType) {
        def generics = objectType.extractChildTypes()
        if (generics.size() != 1) {
            return false
        }

        if (objectType.extractClass() != collectionClass) {
            return false
        }

        return true
    }

    @Override
    boolean canAcceptNbtType(Class nbtType) {
        return nbtType == NbtList
    }

    void supportColletion(Class<T> collectionClass) {
        this.collectionClass = collectionClass
    }

    void createCollectionBy(Supplier<T> factory) {
        this.collectionFactory = factory
    }

    abstract void initialize()
}
