package io.tswf.minecraft.blockventure.core.nbt.mapper

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.INBTObjectMappingProvider
import net.minecraft.nbt.NbtElement

import java.lang.reflect.Type
import java.util.function.Function

@CompileStatic
abstract class AbstractNBTDelegateMappingProvider<ObjectType, NBTType extends NbtElement> implements INBTObjectMappingProvider {

    Class<NBTType> supportedNbtType
    Class<ObjectType> supportedObjectType
    Function<ObjectType, NBTType> toNbtDelegate
    Function<NBTType, ObjectType> fromNbtDelegate

    AbstractNBTDelegateMappingProvider() {
        initialize()
        supportedObjectType.ensureNotNull { new IllegalStateException("There is no supported object type was set for ${this}!") }
        supportedNbtType.ensureNotNull { new IllegalStateException("There is no supported nbt type was set for ${this}!") }
        toNbtDelegate.ensureNotNull { new IllegalStateException("There is no object-to-nbt delegate was set for ${this}!") }
        fromNbtDelegate.ensureNotNull { new IllegalStateException("There is no nbt-to-object delegate was set for ${this}!") }
    }

    @Override
    NbtElement mapObjectToNBT(Object obj, Type objectType) {
        return toNbtDelegate?.apply((ObjectType) obj)
    }

    @Override
    Object mapNBTToObject(NbtElement nbtElement, Type objectType, Object initialInstance) {
        return fromNbtDelegate?.apply((NBTType) nbtElement)
    }

    @Override
    boolean canAcceptObjectType(Type objectType) {
        return supportedObjectType == objectType.extractClass()
    }

    AbstractNBTDelegateMappingProvider<ObjectType, NBTType> supportObject(Class<ObjectType> cl) {
        this.supportedObjectType = cl
        return this
    }

    AbstractNBTDelegateMappingProvider<ObjectType, NBTType> supportNbt(Class<NBTType> cl) {
        this.supportedNbtType = cl
        return this
    }

    AbstractNBTDelegateMappingProvider<ObjectType, NBTType> toNbt(Function<ObjectType, NBTType> fun) {
        this.toNbtDelegate = fun
        return this
    }

    AbstractNBTDelegateMappingProvider<ObjectType, NBTType> fromNbt(Function<NBTType, ObjectType> cl) {
        this.fromNbtDelegate = cl
        return this
    }

    @Override
    boolean canAcceptNbtType(Class nbtType) {
        return nbtType == supportedNbtType
    }

    abstract void initialize()
}
