package io.tswf.minecraft.blockventure.core.nbt.mapper

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.INBTObjectMappingProvider
import net.minecraft.inventory.Inventories
import net.minecraft.item.ItemStack
import net.minecraft.nbt.NbtCompound
import net.minecraft.util.collection.DefaultedList

import java.lang.reflect.Type

@AutoService(INBTObjectMappingProvider)
@CompileStatic
class NBTItemStackDefaultedListMappingProvider extends AbstractNBTDelegateMappingProvider<DefaultedList<ItemStack>, NbtCompound> {

    public static final String STACKS_COUNT_NBT_KEY = "Size"

    @Override
    void initialize() {
        supportObject DefaultedList
        supportNbt NbtCompound
        fromNbt { nbt ->
            def inventorySize = nbt.getInt(STACKS_COUNT_NBT_KEY)

            //TODO: Закостылить так, чтобы в этом случае количество элементов определялось по максимально занятому слоту, м?
            def result = DefaultedList.ofSize(inventorySize, ItemStack.EMPTY)
            Inventories.readNbt(nbt, result)

            return result
        }
        toNbt { stacks ->
            new NbtCompound().let {
                it.putInt(STACKS_COUNT_NBT_KEY, stacks.size())
                Inventories.writeNbt(it, stacks, true)
            }
        }
    }

    @Override
    boolean canAcceptObjectType(Type objectType) {
        def generics = objectType.extractChildTypes()
        if (generics.size() != 1) {
            return false
        }

        if (generics.first().extractClass() != ItemStack) {
            return false
        }

        return super.canAcceptObjectType(objectType)
    }
}
