package io.tswf.minecraft.blockventure.core.network.packet

import com.mojang.blaze3d.systems.RenderSystem
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.netty.buffer.ByteBufInputStream
import io.netty.buffer.ByteBufOutputStream
import io.tswf.minecraft.blockventure.core.api.network.packet.IPlayNetworkPacket
import io.tswf.minecraft.blockventure.core.exception.NetworkPlayPacketSerializationException
import net.minecraft.client.MinecraftClient
import net.minecraft.client.network.ClientPlayNetworkHandler
import net.minecraft.client.util.NetworkUtils
import net.minecraft.network.NetworkThreadUtils
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.listener.PacketListener
import net.minecraft.network.packet.Packet
import net.minecraft.server.network.ServerPlayNetworkHandler

@Slf4j
@CompileStatic
abstract class NetworkPlayPacketWrapper implements Packet<PacketListener> {

    IPlayNetworkPacket packetObject

    NetworkPlayPacketWrapper(IPlayNetworkPacket packetObject) {
        this.packetObject = packetObject
    }

    NetworkPlayPacketWrapper(IPlayNetworkPacket packetObject, DataInputStream data) {
        this(packetObject)
        try {
            packetObject.readFromBytes(data)
        } catch (Throwable e) {
            throw new NetworkPlayPacketSerializationException(e, packetObject)
        }
        // Релизит буфер майн сам
    }

    @Override
    void write(PacketByteBuf buf) {
        new DataOutputStream(new ByteBufOutputStream(buf)).doThenClose { data ->
            try {
                packetObject.writeToBytes(data)
            } catch (Throwable e) {
                throw new NetworkPlayPacketSerializationException(e, packetObject)
            }
        }

        // Релизит буфер майн сам
    }

    @Override
    void apply(PacketListener listener) {
        if (listener instanceof ServerPlayNetworkHandler) {
            def serverWorld = listener.player.serverWorld
            def server = serverWorld.server
            def player = listener.player

            if (!server.isOnThread() && !packetObject.isBackgroundOnServer())  {
                server.executeSync {
                    packetObject.handleOnServer(serverWorld, player)
                }
            } else {
                packetObject.handleOnServer(serverWorld, player)
            }

        } else if (listener instanceof ClientPlayNetworkHandler) {
            def mc = MinecraftClient.getInstance()

            if (!mc.isOnThread() && !packetObject.isBackgroundOnClient())  {
                mc.executeSync {
                    packetObject.handleOnClient(mc.world, mc.player)
                }
            } else {
                packetObject.handleOnClient(mc.world, mc.player)
            }

        } else {
            log.warn("There is unknown packet listener '${listener}' found for packet ${packetObject} with type ${packetObject.class.name}! Skipping handle...")
        }
    }
}
