package io.tswf.minecraft.blockventure.core.network.packet

import groovy.transform.CompileStatic
import groovy.transform.Memoized
import io.netty.buffer.ByteBuf
import io.netty.buffer.ByteBufInputStream
import io.tswf.minecraft.blockventure.core.api.network.packet.IPlayNetworkPacket
import io.tswf.minecraft.blockventure.core.api.registry.PacketRegistry
import io.tswf.minecraft.blockventure.core.exception.InvalidPlayPacketClassException
import io.tswf.minecraft.blockventure.core.exception.NetworkPlayPacketCreationException
import io.tswf.minecraft.blockventure.core.exception.UnknownPacketTypeIdException

import java.lang.reflect.Constructor

@CompileStatic
class NetworkPlayPacketWrapperCreator {

    static NetworkPlayPacketWrapper createWrapperForPlayPacket(ByteBuf buf, boolean server) {
        new DataInputStream(new ByteBufInputStream(buf)).doThenClose { data ->
            createWrapperForPlayPacket(data, server)
        }
    }

    static NetworkPlayPacketWrapper createWrapperForPlayPacket(DataInputStream data, boolean server) {
        def packetId = "<unknown>"

        try {
            packetId = data.readUTF()
            def packetType = PacketRegistry.registeredPacketTypes[packetId]
                    .ensureNotNull { new UnknownPacketTypeIdException(packetId) }

            def packetNoArgsConstructor = getPacketNoArgsConstructor(packetType)
            def packetInstance = (IPlayNetworkPacket) packetNoArgsConstructor.newInstance()

            return server ? new ServerNetworkPlayPacketWrapper(packetInstance, data) : new ClientNetworkPlayPacketWrapper(packetInstance, data)
        } catch (Throwable e) {
            throw new NetworkPlayPacketCreationException(e, packetId)
        }
    }

    //TODO: Вообще если предположить, что во время игры будет меняться класслоадер - может дать утечку кэш по классам
    @Memoized
    static Constructor getPacketNoArgsConstructor(Class packetClass) {
        return packetClass.noArgsConstructor
            .ensureNotNull { new InvalidPlayPacketClassException("There is no required no args constructor", packetClass) }
    }

}
