package io.tswf.minecraft.blockventure.core.network.packet

import groovy.transform.CompileStatic
import net.minecraft.client.MinecraftClient
import net.minecraft.client.network.ClientPlayNetworkHandler
import net.minecraft.nbt.NbtCompound
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.packet.Packet
import net.minecraft.util.math.BlockPos

@CompileStatic
class ServerBlockEntityNbtUpdatedPlayPacket implements Packet<ClientPlayNetworkHandler> {

    BlockPos blockEntityPos
    String blockEntityClassName
    NbtCompound blockEntityNbtData

    ServerBlockEntityNbtUpdatedPlayPacket(PacketByteBuf buf) {
        this.blockEntityNbtData = buf.readNbt()
        this.blockEntityPos = buf.readBlockPos()
        this.blockEntityClassName = buf.readString()
    }

    ServerBlockEntityNbtUpdatedPlayPacket(BlockPos blockEntityPos, String blockEntityClassName, NbtCompound blockEntityNbtData) {
        this.blockEntityPos = blockEntityPos
        this.blockEntityClassName = blockEntityClassName
        this.blockEntityNbtData = blockEntityNbtData
    }

    @Override
    void write(PacketByteBuf buf) {
        buf.writeString(blockEntityClassName)
        buf.writeBlockPos(blockEntityPos)
        buf.writeNbt(blockEntityNbtData)
    }

    @Override
    void apply(ClientPlayNetworkHandler listener) {
        listener.world.getBlockEntity(blockEntityPos)?.let { blockEntityToUpdate ->
            if (blockEntityToUpdate.class.name == blockEntityClassName) {
                def mc = MinecraftClient.getInstance()
                if (mc.isOnThread()) {
                    blockEntityToUpdate.readNbt(blockEntityNbtData)
                } else {
                    mc.executeSync { blockEntityToUpdate.readNbt(blockEntityNbtData) }
                }
            }
        }
    }
}
