package io.tswf.minecraft.blockventure.core.recipe

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.exception.RecipeIngredientObjectParseException
import net.minecraft.recipe.Ingredient

@CompileStatic
class RecipeIngredientObjectParser {

    @Lazy
    static List<IRecipeIngredientObjectParseProvider> ingredientObjectParseProviders = ServiceLoader.load(IRecipeIngredientObjectParseProvider)
        .toList()

    static Ingredient parseIngredientObject(Object ingredientObject) {
        try {
            return ingredientObjectParseProviders.stream()
                    .map {
                        try {
                            it.parseIngredientObject(ingredientObject)
                        } catch (Throwable e) {
                            throw new RecipeIngredientObjectParseException(e, ingredientObject, it)
                        }
                    }
                    .filter { it != null }
                    .firstOrNull()
        } catch (Throwable e) {
            throw new RecipeIngredientObjectParseException(e, ingredientObject, null)
        }
    }
}
