package io.tswf.minecraft.blockventure.core.tag

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.registry.tag.TagPrefixes
import net.minecraft.registry.Registries
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKey
import net.minecraft.registry.tag.TagKey
import net.minecraft.util.Identifier

@CompileStatic
class TagParser {

    public static final String DEFAULT_TAG_PREFIX = TagPrefixes.ITEMS
    public static final String TAG_SEPARATOR = "::"

    public static final Map<String, Registry> PREFIX_TO_REGISTRY = new HashMap<>().tap {
        put(TagPrefixes.ITEMS, Registries.ITEM)
        put(TagPrefixes.BLOCKS, Registries.BLOCK)
    } as Map

    static TagKey parseTag(String tagString) {
        if (tagString == null) {
            return null
        }

        def split = tagString.split(TAG_SEPARATOR, 2)

        def tagName
        def registryPrefix

        if (split.length == 2) {
            registryPrefix = split[0]
            tagName = split[1]
        } else {
            registryPrefix = DEFAULT_TAG_PREFIX
            tagName = split.first()
        }

        registryPrefix = registryPrefix.stripAll()
        tagName = tagName.stripAll()

        def registry = PREFIX_TO_REGISTRY[registryPrefix]
                .ensureNotNull { new IllegalArgumentException("There is no registry for prefix: '${registryPrefix}' in tag string '${tagString}'. Allowed prefixes is: '${PREFIX_TO_REGISTRY.keySet()}'") }

        return TagKey.of((RegistryKey) registry.key, new Identifier(tagName))
    }

}
