package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.block.entity.AutoBlockEntityTicker
import io.tswf.minecraft.blockventure.core.transmutation.expansion.IAutoTickableBlockEntityType
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtHead
import io.tswf.transmutations.transmute.annotation.ReturningValueIfNotNull
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.block.AbstractBlock
import net.minecraft.block.entity.BlockEntityTicker
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.world.World

@CompileStatic
class TransmutationAbstractBlockState {

    @Transmute(
            targetClass = @ClassReference(AbstractBlock.AbstractBlockState),
            targetMethod = @MethodDescription(
                    name = "getBlockEntityTicker",
                    parameterTypeClasses = [ World, BlockEntityType ],
                    returnTypeClass = BlockEntityTicker
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    @ReturningValueIfNotNull
    static BlockEntityTicker injectAutoBlockEntityTicker(World world, BlockEntityType blockEntityType) {
        def autoTickerInfo = (IAutoTickableBlockEntityType) blockEntityType
        if (autoTickerInfo.isAutoTickerEnabled()) {
            return new AutoBlockEntityTicker()
        }

        return null
    }

}
