package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.IReflectiveNBTObject
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import io.tswf.minecraft.blockventure.core.api.registry.BlockEntityRegistry
import io.tswf.minecraft.blockventure.core.nbt.mapper.NBTHelper
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewBlockEntity
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.FieldDescription
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.*
import io.tswf.transmutations.transmute.injection.contexts.InjectionOffset
import io.tswf.transmutations.transmute.injection.points.FieldCallType
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.block.BlockState
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.nbt.NbtCompound
import net.minecraft.util.math.BlockPos

@CompileStatic
class TransmutationBlockEntity {

    public static final String BLOCK_ENTITY_DATA_NBT_KEY = "BlockventureBlockEntityNBT"

    @Transmute(
            targetClass = @ClassReference(BlockEntity),
            targetMethod = @MethodDescription(
                    name = "<init>",
                    parameterTypeClasses = [ BlockEntityType, BlockPos, BlockState ],
                    returnType = @ClassReference(ClassReference.VOID)
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtFieldCall(
            callType = FieldCallType.SET,
            field = @FieldDescription(
                    name = "type",
                    type = @ClassReference(BlockEntityType)
            ),
            owner = @ClassReference(BlockEntity)
    )
    @Offset(InjectionOffset.AFTER)
    static void fetchBlockEntityTypeIfNull(BlockEntityType entityType, BlockPos blockPos, BlockState blockState, @Self BlockEntity self) {
        def viewBlockEntity = (ViewBlockEntity) self
        if (entityType == BlockEntityRegistry.autoSelectType) {
            viewBlockEntity.type = BlockEntityRegistry.findBlockEntityTypeByClass(self.class)
        }
    }


    @Transmute(
            targetClass = @ClassReference(BlockEntity),
            targetMethod = @MethodDescription(
                    name = "readNbt",
                    parameterTypeClasses = NbtCompound,
                    returnType = @ClassReference(ClassReference.VOID)
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    static void readAutoNBT(NbtCompound nbt, @Self BlockEntity self) {
        if (self instanceof IReflectiveNBTObject) {
            def dataCompound = NBTHelper.getOrCreateCompound(nbt, BLOCK_ENTITY_DATA_NBT_KEY)
            NBTObjectMapper.mapToObject(dataCompound, self.class, self)
        }
    }

    @Transmute(
            targetClass = @ClassReference(BlockEntity),
            targetMethod = @MethodDescription(
                    name = "writeNbt",
                    parameterTypeClasses = NbtCompound,
                    returnType = @ClassReference(ClassReference.VOID)
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    static void writeAutoNBT(NbtCompound nbt, @Self BlockEntity self) {
        if (self instanceof IReflectiveNBTObject) {
            def dataCompound = NBTObjectMapper.mapToNbt(self, self.class)
            nbt.put(BLOCK_ENTITY_DATA_NBT_KEY, dataCompound)
        }
    }
}
