package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.block.entity.IAutoSyncableNBTBlockEntity
import io.tswf.minecraft.blockventure.core.network.packet.ServerBlockEntityNbtUpdatedPlayPacket
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewBlockEntity
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtMethodCall
import io.tswf.transmutations.transmute.annotation.LocalVariableValue
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.injection.points.MethodCallInsnType
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.block.entity.BlockEntity
import net.minecraft.nbt.NbtCompound
import net.minecraft.network.packet.Packet
import net.minecraft.server.network.ServerPlayerEntity
import net.minecraft.server.world.ChunkHolder
import net.minecraft.util.math.BlockPos
import net.minecraft.world.World

@CompileStatic
class TransmutationChunkHolder {

    @Transmute(
            targetClass = @ClassReference(ChunkHolder),
            targetMethod = @MethodDescription(
                    name = "sendBlockEntityUpdatePacket",
                    parameterTypeClasses = [ List, World, BlockPos ],
                    returnType = @ClassReference(ClassReference.VOID)
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtMethodCall(
            owner = @ClassReference(BlockEntity),
            method = @MethodDescription(
                    name = "toUpdatePacket",
                    parameterTypeClasses = [],
                    returnTypeClass = Packet
            ),
            callType = MethodCallInsnType.VIRTUAL
    )
    static void notifyAutoSyncBlockEntities(
            List<ServerPlayerEntity> players,
            World world,
            BlockPos pos,
            @LocalVariableValue(index = 4, type = @ClassReference(BlockEntity))
            BlockEntity blockEntity
    ) {
        if (!world.isClient) {
            if (blockEntity instanceof IAutoSyncableNBTBlockEntity) {
                def viewBlockEntity = (ViewBlockEntity) blockEntity
                def updatePacketDataNbt = new NbtCompound()
                blockEntity.onAutoSyncStarted()
                viewBlockEntity.callWriteNbt(updatePacketDataNbt)

                for (def player : players) {
                    player.networkHandler.sendPacket(new ServerBlockEntityNbtUpdatedPlayPacket(pos, blockEntity.class.name, updatePacketDataNbt))
                }

                blockEntity.onAutoSyncCompleted()
            }
        }
    }

}
