package io.tswf.minecraft.blockventure.core.transmutation.transmute

import com.mojang.datafixers.util.Pair
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewDynamicRegistryManagerImmutable
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewMultiNoiseBiomeSourceParameterList
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.common.constant.MethodName
import io.tswf.transmutations.transmute.annotation.AtExit
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.registry.DynamicRegistryManager
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKey
import net.minecraft.registry.RegistryKeys
import net.minecraft.registry.entry.RegistryEntry
import net.minecraft.world.biome.Biome
import net.minecraft.world.biome.source.MultiNoiseBiomeSourceParameterList
import net.minecraft.world.biome.source.util.MultiNoiseUtil

import java.util.stream.Stream

@Slf4j
@CompileStatic
class TransmutationDynamicRegistryManagerImmutable {

    static void injectIntoDynamicRegistries(Map<RegistryKey, Registry<?>> registriesMap) {
        def multipleNoiseListRegistry = (Registry<MultiNoiseBiomeSourceParameterList>) registriesMap[RegistryKeys.MULTI_NOISE_BIOME_SOURCE_PARAMETER_LIST]
        def biomeRegistry = (Registry<Biome>) registriesMap[RegistryKeys.BIOME]

        if (biomeRegistry != null && multipleNoiseListRegistry != null) {
            def allNoiseBiomesLists = multipleNoiseListRegistry.toList()
            for (def noiseBiomesList : allNoiseBiomesLists) {
                def viewNoiseBiomesList = (ViewMultiNoiseBiomeSourceParameterList) noiseBiomesList

                def (regEntries, directEntries) = noiseBiomesList.entries.entries.splitBy { it.second.key.isPresent() }

                def updatedDirectEntries = directEntries.toList()
                updatedDirectEntries.clear()

                for (def directEntry : directEntries) {
                    def biomeValue = directEntry.second.value()
                    def biomeKey = biomeRegistry.getKey(biomeValue).orNull()

                    if (biomeKey == null) {
                        updatedDirectEntries << directEntry
                    } else {
                        updatedDirectEntries << Pair.of(directEntry.first, biomeRegistry.getEntry(biomeValue))
                    }
                }

                def newEntries = new MultiNoiseUtil.Entries(regEntries + updatedDirectEntries)
                viewNoiseBiomesList.entries = newEntries
            }
        }
//        log.info("Start biomes enrich...")
//        registriesMap[RegistryKeys.BIOME]?.let { registry ->
//            def biomeRegistry = (Registry<Biome>) registry
//
//            biomeRegistry.entrySet.stream()
//                .map { it.value }
//                .forEach {
//                    BiomeEnricher.enrichBiome(it)
//                }
//        }
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ List ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(List registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries


        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ Map ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Map registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ Stream ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Stream registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

}
