package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.item.IItemGroupStacksProvider
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtExit
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.item.ItemGroup
import net.minecraft.item.ItemGroup.DisplayContext
import net.minecraft.registry.Registries

@CompileStatic
class TransmutationItemGroup {

    @Transmute(
            targetClass = @ClassReference(ItemGroup),
            targetMethod = @MethodDescription(name = "updateEntries", descriptor = '(Lnet/minecraft/item/ItemGroup$DisplayContext;)V'),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void registerEntries(DisplayContext displayContext, @Self ItemGroup self) {
        Registries.ITEM.stream()
                .forEach {
                    if (it instanceof IItemGroupStacksProvider) {
                        if (Registries.ITEM_GROUP.getKey(self).orNull() in it.itemGroupKeys) {
                            it.addStacksToGroup(self, self.displayStacks)
                            it.addStacksToGroup(self, self.searchTabStacks)
                        }
                    }
                }
    }
}
