package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.world.biome.BiomeEnrichService
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtHead
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKeys
import net.minecraft.registry.ServerDynamicRegistryType
import net.minecraft.server.MinecraftServer
import net.minecraft.world.biome.Biome

@Slf4j
@CompileStatic
class TransmutationMinecraftServer {

    @Transmute(
            targetClass = @ClassReference(MinecraftServer),
            targetMethod = @MethodDescription(
                    name = "loadWorld",
                    parameterTypeClasses = [],
                    returnTypeClass = ClassReference.VOID
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtHead
    static void enrichBiomesOnServerLoad(@Self MinecraftServer server) {
        def biomeRegistry = server.combinedDynamicRegistries.get(ServerDynamicRegistryType.WORLDGEN)
            .streamAllRegistries()
                .filter { it.key() == RegistryKeys.BIOME }
                .map { (Registry<Biome>) it.value() }
                .firstNotNull()

        log.info("Start biomes enriching for server ${server}...")

        def enrichedBiomesCount = biomeRegistry.stream()
            .filter {
                BiomeEnrichService.enrichBiome(it)
            }
            .count()

        log.info("Complete biomes enriching, total ${enrichedBiomesCount} biomes updated!")
    }

}
