package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.network.packet.ClientNetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.network.packet.NetworkPlayPacketWrapperCreator
import io.tswf.minecraft.blockventure.core.network.packet.ServerBlockEntityNbtUpdatedPlayPacket
import io.tswf.minecraft.blockventure.core.network.packet.ServerNetworkPlayPacketWrapper
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewNetworkState
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewNetworkStatePacketHandler
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.FieldDescription
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtFieldCall
import io.tswf.transmutations.transmute.annotation.Offset
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.injection.contexts.InjectionOffset
import io.tswf.transmutations.transmute.injection.points.FieldCallType
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.network.NetworkSide
import net.minecraft.network.NetworkState
import net.minecraft.network.PacketByteBuf
import net.minecraft.network.packet.Packet

import java.util.function.Function

@Slf4j
@CompileStatic
class TransmutationNetworkState {

    @Transmute(
            targetClass = @ClassReference(NetworkState),
            targetMethod = @MethodDescription(
                    name = "<clinit>",
                    descriptor = "()V"
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtFieldCall(
            callType = FieldCallType.STATIC_SET,
            field = @FieldDescription(
                    type = @ClassReference(NetworkState),
                    name = "PLAY"
            ),
            owner = @ClassReference(NetworkState)
    )
    @Offset(InjectionOffset.AFTER)
    static void registerBlockventurePlayNetworkPackets() {
        def playNetworkState = NetworkState.PLAY
        def viewPlayNetworkState = (ViewNetworkState) (Object) playNetworkState
        def packetHandlers = (Map<NetworkSide, ViewNetworkStatePacketHandler>) (Map) viewPlayNetworkState.packetHandlers

        for (def networkSide : NetworkSide.values()) {
            packetHandlers[networkSide]?.let { viewPacketHandler ->
                log.info("Registering ${ClientNetworkPlayPacketWrapper.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        ClientNetworkPlayPacketWrapper as Class,
                        { PacketByteBuf buf -> NetworkPlayPacketWrapperCreator.createWrapperForPlayPacket(buf, false) } as Function<PacketByteBuf, Packet>
                )

                log.info("Registering ${ServerNetworkPlayPacketWrapper.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        ServerNetworkPlayPacketWrapper as Class,
                        { PacketByteBuf buf -> NetworkPlayPacketWrapperCreator.createWrapperForPlayPacket(buf, true) } as Function<PacketByteBuf, Packet>
                )

                log.info("Registering ${ServerBlockEntityNbtUpdatedPlayPacket.name} packet on side ${networkSide.name()}!")

                viewPacketHandler.callRegister(
                        ServerBlockEntityNbtUpdatedPlayPacket as Class,
                        { PacketByteBuf buf -> new ServerBlockEntityNbtUpdatedPlayPacket(buf) } as Function<PacketByteBuf, Packet>
                )
            }
        }
    }
}
