package io.tswf.minecraft.blockventure.core.transmutation.transmute

import com.google.gson.JsonElement
import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.recipe.RecipeRegistry
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewRecipeManager
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtExit
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.recipe.RecipeManager
import net.minecraft.recipe.RecipeType
import net.minecraft.resource.ResourceManager
import net.minecraft.util.Identifier
import net.minecraft.util.profiler.Profiler

@CompileStatic
class TransmutationRecipeManager {

    @Transmute(
            targetClass = @ClassReference(RecipeManager),
            targetMethod = @MethodDescription(name = "apply", parameterTypeClasses = [Map, ResourceManager, Profiler], returnType = @ClassReference(ClassReference.VOID)),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void apply(Map<Identifier, JsonElement> map2, ResourceManager arg, Profiler arg2, @Self RecipeManager self) {
        def viewRecipeManager = (ViewRecipeManager) self

        // По-умолчанию там ImmutableMap, так что раскрываем это в карту, которую можно мутировать
        def mutableRecipes = new HashMap<>(viewRecipeManager.recipes)
        def mutableRecipesById = new HashMap<>(viewRecipeManager.recipesById)
        def mutableShapedRecipes = new HashMap<>(mutableRecipes.get(RecipeType.CRAFTING))
        mutableRecipes[RecipeType.CRAFTING] = mutableShapedRecipes

        def mutableSmeltingRecipes = new HashMap<>(mutableRecipes.get(RecipeType.SMELTING))
        mutableRecipes[RecipeType.SMELTING] = mutableSmeltingRecipes

        for (def recipeEntry : RecipeRegistry.registeredShapedRecipes) {
            def recipeKey = new Identifier(recipeEntry.key)
            mutableShapedRecipes[recipeKey] = recipeEntry.value
            mutableRecipesById[recipeKey] = recipeEntry.value
        }

        for (def recipeEntry : RecipeRegistry.registeredShapelessRecipe) {
            def recipeKey = new Identifier(recipeEntry.key)
            mutableShapedRecipes[recipeKey] = recipeEntry.value
            mutableRecipesById[recipeKey] = recipeEntry.value
        }

        for (def recipeEntry : RecipeRegistry.registeredSmeltingRecipes) {
            def recipeKey = new Identifier(recipeEntry.key)
            mutableSmeltingRecipes[recipeKey] = recipeEntry.value
            mutableRecipesById[recipeKey] = recipeEntry.value
        }

        viewRecipeManager.setRecipes(mutableRecipes)
        viewRecipeManager.setRecipesById(mutableRecipesById)
    }
}
