package io.tswf.minecraft.blockventure.core.transmutation.transmute

import com.mojang.datafixers.util.Pair
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.api.registry.BiomeRegistry
import io.tswf.minecraft.blockventure.core.api.registry.Register
import io.tswf.minecraft.blockventure.core.world.biome.DimensionBiomesEnrichService
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.transmute.annotation.AtMethodCall
import io.tswf.transmutations.transmute.annotation.LocalVariableValue
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.injection.points.MethodCallInsnType
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKey
import net.minecraft.registry.RegistryKeys
import net.minecraft.registry.RegistryLoader

@Slf4j
@CompileStatic
class TransmutationRegistryLoader {

    @Transmute(
            targetClass = @ClassReference(RegistryLoader),
            targetMethod = @MethodDescription(
                    // Лямбда, которая передается в forEach
                    name = "method_45128",
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ Map, Pair ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtMethodCall(
            callType = MethodCallInsnType.INTERFACE,
            owner = @ClassReference(Registry),
            method = @MethodDescription(
                    name = "freeze",
                    parameterTypeClasses = [],
                    returnType = @ClassReference(Registry)
            )
    )
    static void injectIntoDynamicRegistries(
            Map unusedMap,
            Pair unusedPair,

            @LocalVariableValue(index = 2, type = @ClassReference(Registry))
            Registry registry
    ) {
        if (registry.key == RegistryKeys.MULTI_NOISE_BIOME_SOURCE_PARAMETER_LIST) {
            DimensionBiomesEnrichService.enrichDimensionBiomesByRegistry(registry)
        }

        if (registry.key == RegistryKeys.BIOME) {
            for (def biomeToRegisterEntry : BiomeRegistry.registeredBiomes) {
                log.info("Registering ${biomeToRegisterEntry.value} biome by ${biomeToRegisterEntry.key} id!")
                Register.register(registry, biomeToRegisterEntry.value, biomeToRegisterEntry.key)
            }
        }
    }

}
